{% load i18n %}

{# Edit form #}
<form class="editable-form" method="post" action="{% url edit %}" id="{{ uuid }}">
    {% for field in form %}{{ field }}{% endfor %}<br />
    <input type="submit" value="{% trans "Save" %}" class="button" />
    <input type="button" value="{% trans "Cancel" %}" class="button" />
</form>

{# Original content wrapped in span #}
<div class="editable-original">{{ original }}</div>

{# Edit link #}
<a class="editable-link" href="#" rel="#{{ uuid }}">{% trans "Edit" %}</a>

{# Edit highlight #}
<div class="editable-highlight"></div>

